@@ -9,27 +9,38 @@ import logging
 import os
 import threading
 
-from dataclasses import asdict, dataclass
-from typing import Callable, Dict, Generic, Iterator, List, Optional
+from collections import Counter
+from dataclasses import asdict, dataclass, field
+from threading import Thread
+from time import sleep
+from typing import Callable, Dict, Generic, Iterator, List, Optional, Tuple
 
 import requests
 
-from backports.cached_property import cached_property
-
 import gretel_client.agents.agent_telemetry as telemetry
 
 from gretel_client.agents.drivers.driver import ComputeUnit, Driver
 from gretel_client.agents.drivers.registry import get_driver
 from gretel_client.agents.logger import configure_logging
-from gretel_client.config import configure_custom_logger, get_session_config
+from gretel_client.config import (
+    ClientConfig,
+    configure_custom_logger,
+    get_session_config,
+    RunnerMode,
+)
 from gretel_client.docker import CloudCreds, DataVolumeDef
 from gretel_client.helpers import do_api_call
 from gretel_client.projects import get_project
-from gretel_client.rest.apis import JobsApi, ProjectsApi, UsersApi
+from gretel_client.projects.exceptions import GretelProjectError
+from gretel_client.rest.api.users_api import UsersApi
+from gretel_client.rest.apis import JobsApi, ProjectsApi
 
 configure_logging()
 
 
+_CLUSTERID_HEADER_KEY = "X-Gretel-Clusterid"
+
+
 class AgentError(Exception):
     ...
 
@@ -44,15 +55,38 @@ class AgentConfig:
     max_workers: int = 1
     """The max number of workers the agent instance will launch."""
 
+    project_retry_limit: int = 10
+    """The max number of times to retry resolving a project."""
+
+    _max_workers_from_config: int = 0
+    """The max workers that were passed in via the config"""
+
     log_factory: Callable = lambda _: None
     """A factory function to ship worker logs to. If none is provided
     log messages from the worker will be suppressed, though they still
     be logged to their respective artifact endpoint.
     """
 
-    project: Optional[str] = None
-    """Determines the project to pull jobs for. If none if provided, than
-    jobs from all projects will be fetched.
+    projects: List[str] = field(default_factory=list)
+    """Determines the projects to pull jobs for. If none is provided, then
+    jobs from all projects will be fetched. As a special value, a list with
+    the single value "all" will instruct the agent to fetch jobs from all
+    projects that the user has at least admin access to.
+    """
+
+    org_only: bool = field(default=False)
+    """Limits the agent to dispatching jobs in the same organization (enterprise/team)
+    as the user running the agent only. If the user is not part of an organization,
+    only jobs owned by the user will be processed. For security reasons, it is
+    recommended to set this option to True.
+    """
+
+    auto_accept_project_invites: bool = field(default=False)
+    """If set to true, the agent will check for and accept pending project invites
+    originating from the same organization before polling for jobs. This is useful
+    in conjunction with the projects = ["all"] setting mentioned above, to allow
+    users to make their projects available to the agent without having to manually
+    accept invites.
     """
 
     creds: Optional[List[CloudCreds]] = None
@@ -64,6 +98,9 @@ class AgentConfig:
     disable_cloud_logging: bool = False
     """Disable sending worker logs to the cloud"""
 
+    disable_cloud_report_scores: bool = False
+    """Disable sending model quality report scores to the cloud"""
+
     volumes: Optional[List[DataVolumeDef]] = None
     """A list of volumes to mount into the worker container"""
 
@@ -76,44 +113,60 @@ class AgentConfig:
     enable_prometheus: bool = False
     """Sets up the prometheus endpoint for the running agent"""
 
-    _max_runtime_seconds: Optional[int] = None
-    """TODO: implement"""
+    runner_modes: Optional[List[RunnerMode]] = None
+
+    cluster_guid: Optional[str] = None
+
+    session: Optional[ClientConfig] = None
+
+    _project_ids: Optional[List[str]] = None
+    """Cached project ID."""
+
+    _failed_projects: Counter = field(default_factory=Counter)
+    """Projects that we have failed to resolve."""
 
     def __post_init__(self):
-        if not self._max_runtime_seconds:
-            self._max_runtime_seconds = self._lookup_max_runtime()
-        if self.max_workers > 1:
-            max_jobs_active = self._lookup_max_jobs_active()
-            if max_jobs_active < self.max_workers:
-                logger = logging.getLogger(__name__)
-                logger.warning(
-                    "Max workers value supplied via CLI: '%d' "
-                    "is greater than the max allowed by your account: '%d'",
-                    self.max_workers,
-                    max_jobs_active,
-                )
-                logger.warning("Setting max workers to be %d", max_jobs_active)
-                self.max_workers = max_jobs_active
+        if self.session is None:
+            self.session = get_session_config()
+        self._logger = logging.getLogger(__name__)
+        self._max_workers_from_config = self.max_workers
+        self._update_max_workers()
 
-    @property
-    def max_runtime_seconds(self) -> int:
-        if not self._max_runtime_seconds:
-            raise AgentError("Could not fetch user config. Please restart the agent.")
-        return self._max_runtime_seconds
+    def _update_max_workers(self) -> int:
+        initial_value = self.max_workers
 
-    def _lookup_max_runtime(self) -> int:
-        user_api = get_session_config().get_api(UsersApi)
-        return (
-            user_api.users_me()
-            .get("data")
-            .get("me")
-            .get("service_limits")
-            .get("max_job_runtime")
+        max_jobs_active = self._lookup_max_jobs_active()
+
+        self.max_workers = (
+            max_jobs_active
+            if self._max_workers_from_config <= 0
+            else min(max_jobs_active, self._max_workers_from_config)
         )
 
+        if (
+            self._max_workers_from_config > 0
+            and max_jobs_active < self._max_workers_from_config
+        ):
+            self._logger.warning(
+                "Max workers set by config (%d) higher than the value from the API (%d)",
+                self._max_workers_from_config,
+                max_jobs_active,
+            )
+
+        if initial_value != self.max_workers:
+            self._logger.info(
+                "Max workers set to %d",
+                self.max_workers,
+            )
+
+        telemetry.set_max_workers(
+            max_workers=self.max_workers, previous_max_workers=initial_value
+        )
+        return self.max_workers
+
     def _lookup_max_jobs_active(self) -> int:
         try:
-            resp = do_api_call("GET", "/users/me/billing")
+            resp = do_api_call("GET", "/users/me/billing", session=self.session)
             return (
                 resp.get("billing")
                 .get("me")
@@ -133,10 +186,66 @@ class AgentConfig:
     def as_dict(self) -> dict:
         return asdict(self)
 
-    @cached_property
-    def project_id(self) -> str:
-        project = get_project(name=self.project)
-        return project.project_id
+    @property
+    def project_ids(self) -> List[str]:
+        if not self.projects:
+            return []
+        # Special case: single-element list with just "all" means to query jobs from
+        # all projects with sufficient access to, regardless of job or project ownership.
+        if self.projects == ["all"]:
+            return self.projects
+        self._check_cache()
+        if not self._project_ids:
+            # Need to throw in this scenario to avoid not filtering by at least one project.
+            raise GretelProjectError("unable to resolve any supplied projects")
+        return self._project_ids
+
+    def invalidate_project_ids(self) -> None:
+        self._project_ids = None
+
+    def _check_cache(self):
+        if self._project_ids and len(self._failed_projects) == 0:
+            # Cache has all needed values.
+            return
+
+        # If cache is empty, resolve everything. Otherwise, just resolve previously failed projects.
+        if not self._project_ids:
+            self._project_ids = []
+            name_id_pairs = self._resolve_project_ids(self.projects)
+        else:
+            name_id_pairs = self._resolve_project_ids(
+                list(self._failed_projects.keys())
+            )
+        for project_name, project_id in name_id_pairs:
+            if project_id:
+                self._project_ids.append(project_id)
+                self._failed_projects.pop(project_name, 0)
+            else:
+                self._failed_projects[project_name] += 1
+
+        # Throw if values overrun the retry limit.
+        failures_over_limit = [
+            x
+            for x, count in self._failed_projects.items()
+            if count > self.project_retry_limit
+        ]
+        if len(failures_over_limit) > 0:
+            raise GretelProjectError(
+                "projects (%s) have exceeded the resolution retry limit",
+                ", ".join(failures_over_limit),
+            )
+
+    def _resolve_project_ids(
+        self, projects_to_resolve: List[str]
+    ) -> List[Tuple[str, Optional[str]]]:
+        resolved = []
+        for project_name in projects_to_resolve:
+            try:
+                project = get_project(name=project_name, session=self.session)
+                resolved.append((project_name, project.project_id))
+            except GretelProjectError:
+                resolved.append((project_name, None))
+        return resolved
 
 
 @dataclass
@@ -152,13 +261,14 @@ class Job:
     job_type: str
     container_image: str
     worker_token: str
-    max_runtime_seconds: int
     instance_type: str
     log: Optional[Callable] = None
     cloud_creds: Optional[List[CloudCreds]] = None
     artifact_endpoint: Optional[str] = None
     disable_cloud_logging: bool = False
+    disable_cloud_report_scores: bool = False
     env_vars: Optional[dict] = None
+    session: Optional[ClientConfig] = None
 
     @classmethod
     def from_dict(cls, source: dict, agent_config: AgentConfig) -> Job:
@@ -171,11 +281,12 @@ class Job:
             log=agent_config.log_factory(
                 source.get("run_id") or source.get("model_id")
             ),
-            max_runtime_seconds=agent_config.max_runtime_seconds,
             cloud_creds=agent_config.creds,
             artifact_endpoint=agent_config.artifact_endpoint,
             disable_cloud_logging=agent_config.disable_cloud_logging,
+            disable_cloud_report_scores=agent_config.disable_cloud_report_scores,
             env_vars=agent_config.env_vars,
+            session=agent_config.session,
         )
 
     @property
@@ -185,6 +296,8 @@ class Job:
             params["--artifact-endpoint"] = self.artifact_endpoint
         if self.disable_cloud_logging:
             params["--disable-cloud-logging"] = ""
+        if self.disable_cloud_report_scores:
+            params["--disable-cloud-report-scores"] = ""
         return params
 
     @property
@@ -207,11 +320,11 @@ class Job:
 
     @property
     def gretel_stage(self) -> str:
-        return get_session_config().stage
+        return self.session.stage
 
     @property
     def gretel_endpoint(self) -> str:
-        return get_session_config().endpoint
+        return self.session.endpoint
 
     @property
     def needs_gpu(self) -> bool:
@@ -221,13 +334,34 @@ class Job:
 class RateLimiter:
     """Limits the amount of jobs the agent can place."""
 
-    def __init__(self, max_active_jobs: int, job_manager: JobManager):
+    def __init__(
+        self, max_active_jobs: int, job_manager: JobManager, agent_config: AgentConfig
+    ):
         self._job_manger = job_manager
         self._max_active_jobs = max_active_jobs
+        self._agent_config = agent_config
+        self._logger = logging.getLogger(__name__)
+        self._poll_interval_secs = 300
 
     def has_capacity(self) -> bool:
         return self._job_manger.active_jobs < self._max_active_jobs
 
+    def update_max_active_jobs(self):
+        previous = self._max_active_jobs
+        new_value = self._agent_config._update_max_workers()
+        if previous != new_value:
+            self._max_active_jobs = new_value
+
+    def update_max_active_jobs_loop(self):
+        while True:
+            try:
+                self.update_max_active_jobs()
+                sleep(self._poll_interval_secs)
+            except Exception:
+                self._logger.warning(
+                    "Error checking max active jobs endpoints", exc_info=True
+                )
+
 
 class JobManager(Generic[ComputeUnit]):
     """Responsible for tracking the status of jobs as they are
@@ -248,12 +382,17 @@ class JobManager(Generic[ComputeUnit]):
         telemetry.increase_active_jobs()
 
     def _update_active_jobs(self) -> None:
-        for job in list(self._active_jobs):
-            if not self._driver.active(self._active_jobs[job]):
-                self._logger.info(f"Job {job} completed")
-                self._driver.clean(self._active_jobs[job])
-                self._active_jobs.pop(job)
-                telemetry.decrease_active_jobs()
+        for job_name, job in list(self._active_jobs.items()):
+            if self._driver.active(job):
+                continue
+            if self._driver.has_errored(job):
+                self._logger.error("Job %s errored out", job_name)
+                # Intentionally leave Job object for debugging, will eventually be garbage collected
+            else:
+                self._logger.info("Job %s completed/ended", job_name)
+                self._driver.clean(job)
+            self._active_jobs.pop(job_name)
+            telemetry.decrease_active_jobs()
 
     def contains_job(self, job: Job) -> bool:
         return job.uid in self._active_jobs
@@ -288,22 +427,43 @@ class Poller(Iterator):
     max_wait_secs = 16
 
     def __init__(
-        self, jobs_api: JobsApi, rate_limiter: RateLimiter, agent_config: AgentConfig
+        self,
+        jobs_api: JobsApi,
+        rate_limiter: RateLimiter,
+        agent_config: AgentConfig,
     ):
         self._agent_config = agent_config
         self._jobs_api = jobs_api
         self._rate_limiter = rate_limiter
         self._logger = logging.getLogger(__name__)
         self._interrupt = threading.Event()
+        self._runner_modes = agent_config.runner_modes or [RunnerMode.MANUAL]
+        self._last_invite_check_time = None
 
     def __iter__(self):
         return self
 
+    @property
+    def session(self) -> ClientConfig:
+        return self._agent_config.session
+
     def interrupt(self):
         return self._interrupt.set()
 
     def poll_endpoint(self) -> Optional[Job]:
-        next_job = self._jobs_api.receive_one(project_id=self._agent_config.project_id)
+        api_kwargs = {}
+        project_ids = self._agent_config.project_ids
+        if len(project_ids) > 0:
+            api_kwargs["project_ids"] = project_ids
+        if self._agent_config.org_only:
+            api_kwargs["org_only"] = True
+        if self._runner_modes:
+            api_kwargs["runner_modes"] = [
+                runner_mode.value for runner_mode in self._runner_modes
+            ]
+        if self._agent_config.cluster_guid:
+            api_kwargs["cluster_guid"] = self._agent_config.cluster_guid
+        next_job = self._jobs_api.receive_one(**api_kwargs)
         if next_job["data"]["job"] is not None:
             return Job.from_dict(next_job["data"]["job"], self._agent_config)
 
@@ -312,12 +472,22 @@ class Poller(Iterator):
         while True and not self._interrupt.is_set():
             if self._rate_limiter.has_capacity():
                 job = None
+                if self._agent_config.auto_accept_project_invites:
+                    try:
+                        self._check_project_invites()
+                    except Exception:
+                        self._logger.warning(
+                            "Could not check for pending project invites.",
+                            exc_info=True,
+                        )
                 try:
                     job = self.poll_endpoint()
-                except Exception as ex:
+                except Exception:
                     self._logger.warning(
-                        f"There was a problem calling the jobs endpoint {ex}"
+                        "There was a problem calling the jobs endpoint.",
+                        exc_info=True,
                     )
+                    self._agent_config.invalidate_project_ids()
                 if job:
                     return job
             self._interrupt.wait(wait_secs)
@@ -327,6 +497,35 @@ class Poller(Iterator):
             else:
                 wait_secs += wait_secs**2
 
+    def _check_project_invites(self):
+        users_api = self.session.get_api(UsersApi)
+
+        kwargs = {
+            "invite_types": ["project"],
+            "org_only": True,
+            "include_expired": False,
+        }
+        if self._last_invite_check_time:
+            kwargs["since"] = self._last_invite_check_time
+
+        resp = users_api.get_users_me_invites(**kwargs)
+        resp_data = resp.get("data", {})
+        for invite in resp_data.get("invites", []):
+            project_guid = invite["project_guid"]
+            inviter = invite.get("invited_by", {}).get("email", "unknown")
+            self._logger.info(
+                f"Accepting invite to project {project_guid} by {inviter}"
+            )
+            if invite.get("level", 0) < 3:
+                self._logger.warn(
+                    "Warning: invite does not grant administrator access, some features might not work"
+                )
+            users_api.accept_users_me_invites(invite_id=invite["_id"])
+
+        self._last_invite_check_time = resp_data.get(
+            "timestamp", self._last_invite_check_time
+        )
+
 
 class Agent:
     """Starts an agent"""
@@ -335,25 +534,44 @@ class Agent:
         self._logger = logging.getLogger(__name__)
         configure_custom_logger(self._logger)
         self._config = config
-        self._client_config = get_session_config()
+        self._client_config = config.session
         self._driver = get_driver(config)
         self._jobs_manager = JobManager(self._driver)
-        self._rate_limiter = RateLimiter(config.max_workers, self._jobs_manager)
-        self._jobs_api = self._client_config.get_api(JobsApi)
-        self._projects_api = self._client_config.get_api(ProjectsApi)
-        self._jobs = Poller(self._jobs_api, self._rate_limiter, self._config)
+        self._rate_limiter = RateLimiter(config.max_workers, self._jobs_manager, config)
+
+        default_headers = None
+        if config.cluster_guid:
+            default_headers = {_CLUSTERID_HEADER_KEY: config.cluster_guid}
+        self._jobs_api = self._client_config.get_api(
+            JobsApi, default_headers=default_headers
+        )
+        self._projects_api = self._client_config.get_api(
+            ProjectsApi, default_headers=default_headers
+        )
+        self._jobs = Poller(
+            self._jobs_api,
+            self._rate_limiter,
+            self._config,
+        )
         self._interrupt = threading.Event()
 
-    def start(self, cooloff: float = 5):
+    def start(self):
+        """Start the agent"""
         if self._config.enable_prometheus:
             self._logger.info("Enabling prometheus client")
             telemetry.setup_prometheus()
-            telemetry.set_config_metrics(
+            telemetry.set_max_workers(
                 max_workers=self._config.max_workers,
-                max_runtime_seconds=self._config.max_runtime_seconds,
             )
-        """Start the agent"""
         self._logger.info("Agent started, waiting for work to arrive")
+        if self._config.cluster_guid:
+            self._logger.info("Cluster ID is %s", self._config.cluster_guid)
+
+        thread = Thread(
+            target=self._rate_limiter.update_max_active_jobs_loop, daemon=True
+        )
+        thread.start()
+
         for job in self._jobs:
             if not job:
                 if self._interrupt.is_set():
@@ -375,16 +593,14 @@ class Agent:
                 telemetry.increment_job_count()
                 self._update_job_status(job)
 
-            # todo: add in read lock to jobs endpoint. this sleep is
-            # a stopgap until then. without this the agent is going to
-            # try and launch multiple containers of the same job.
-            self._interrupt.wait(cooloff)
-
     def _update_job_status(self, job: Job) -> None:
         try:
+            self._logger.info("Updating status to Pending for job %s", job.uid)
             worker_json = base64.standard_b64decode(job.worker_token).decode("ascii")
             worker_key = json.loads(worker_json)["model_key"]
             headers = {"Authorization": worker_key}
+            if self._config.cluster_guid:
+                headers[_CLUSTERID_HEADER_KEY] = self._config.cluster_guid
             url = f"{job.gretel_endpoint}/projects/models"
             params = {"uid": job.uid, "type": job.job_type}
             data = {"uid": job.uid, "status": "pending"}
@@ -394,6 +610,7 @@ class Agent:
                 headers=headers,
                 json=data,
                 params=params,
+                timeout=15,
             )
             self._logger.debug(resp.text)
             resp.raise_for_status()
