@@ -7,8 +7,8 @@ import click
 
 from gretel_client.agents.agent import AgentConfig, get_agent
 from gretel_client.agents.drivers.driver import GPU
-from gretel_client.cli.common import pass_session, project_option, SessionContext
-from gretel_client.config import get_session_config
+from gretel_client.cli.common import pass_session, SessionContext
+from gretel_client.config import get_session_config, RunnerMode
 from gretel_client.docker import AwsCredFile, CaCertFile, check_gpu, DataVolumeDef
 
 
@@ -35,7 +35,29 @@ def build_logger(job_id: str) -> Callable:
 @click.option(
     "--max-workers", metavar="COUNT", help="Max number of workers to launch.", default=2
 )
-@project_option
+@click.option(
+    "--project",
+    allow_from_autoenv=True,
+    envvar="GRETEL_DEFAULT_PROJECT",
+    help="CSV of Gretel projects to execute command from.",
+    metavar="NAME",
+)
+@click.option(
+    "--same-org-only",
+    is_flag=True,
+    envvar="GRETEL_SAME_ORG_ONLY",
+    allow_from_autoenv=True,
+    help="If this is set, only jobs from the same organization as the running user will be executed.",
+    type=bool,
+)
+@click.option(
+    "--auto-accept-project-invites",
+    is_flag=True,
+    envvar="GRETEL_AGENT_AUTO_ACCEPT_PROJECT_INVITES",
+    allow_from_autoenv=True,
+    help="If this is set, the Gretel Agent will automatically check for and accept project invites originating from within the organization.",
+    type=bool,
+)
 @click.option(
     "--aws-cred-path",
     metavar="PATH",
@@ -48,6 +70,14 @@ def build_logger(job_id: str) -> Callable:
     default=None,
     envvar="GRETEL_ARTIFACT_ENDPOINT",
 )
+@click.option(
+    "--runner-modes",
+    metavar="RUNNER_MODES",
+    help="Runner modes used to poll the jobs endpoint",
+    default=None,
+    envvar="RUNNER_MODES",
+    multiple=True,
+)
 @click.option(
     "--env",
     metavar="KEY=VALUE",
@@ -70,24 +100,39 @@ def build_logger(job_id: str) -> Callable:
     help="Disable sending worker logs to Gretel Cloud.",
     default=False,
 )
+@click.option(
+    "--disable-cloud-report-scores",
+    help="Disable sending model report scores to Gretel Cloud.",
+    default=False,
+)
 @click.option(
     "--enable-prometheus",
     help="Enable the prometheus metrics endpoint on port 8080",
     default=False,
 )
+@click.option(
+    "--cluster-id",
+    help="Hybrid Cluster ID to operate on",
+    envvar="GRETEL_CLUSTER_ID",
+)
 @pass_session
 def start(
     sc: SessionContext,
     driver: str,
     max_workers: int,
-    project: str = None,
-    aws_cred_path: str = None,
-    artifact_endpoint: str = None,
-    env: List[str] = None,
-    volume: List[str] = None,
+    project: Optional[str] = None,
+    same_org_only: bool = False,
+    auto_accept_project_invites: bool = False,
+    aws_cred_path: Optional[str] = None,
+    artifact_endpoint: Optional[str] = None,
+    env: Optional[List[str]] = None,
+    volume: Optional[List[str]] = None,
     ca_bundle: Optional[str] = None,
     disable_cloud_logging: bool = False,
+    disable_cloud_report_scores: bool = False,
     enable_prometheus: bool = False,
+    runner_modes: Optional[List[str]] = None,
+    cluster_id: Optional[str] = None,
 ):
     sc.log.info(f"Starting Gretel agent using driver {driver}.")
     creds = []
@@ -117,19 +162,36 @@ def start(
             sc.log.info("GPU found.")
         else:
             sc.log.info("No GPU found. Continuing without one.")
+    runner_modes_as_enum = None
+    if runner_modes:
+        runner_modes_as_enum = [
+            RunnerMode.parse(runner_mode_str) for runner_mode_str in runner_modes
+        ]
+        if RunnerMode.CLOUD in runner_modes_as_enum:
+            raise ValueError("'cloud' is not a legal runner mode for the agent")
+
+    projects = []
+    if project:
+        projects = [p.strip() for p in project.split(",")]
 
     config = AgentConfig(
-        project=project,
+        projects=projects,
+        org_only=same_org_only,
+        auto_accept_project_invites=auto_accept_project_invites,
         max_workers=max_workers,
         driver=driver,
         creds=creds,
         log_factory=build_logger,
         artifact_endpoint=artifact_endpoint,
         disable_cloud_logging=disable_cloud_logging,
+        disable_cloud_report_scores=disable_cloud_report_scores,
         env_vars=env_dict,
         volumes=volumes,
         capabilities=capabilities,
         enable_prometheus=enable_prometheus,
+        runner_modes=runner_modes_as_enum,
+        cluster_guid=cluster_id,
+        session=sc.session,
     )
     agent = get_agent(config)
     sc.register_cleanup(lambda: agent.interrupt())
