@@ -4,8 +4,8 @@ from gretel_client.cli.agent import agent
 from gretel_client.cli.artifacts import artifacts
 from gretel_client.cli.common import pass_session, SessionContext
 from gretel_client.cli.connections import connections
-from gretel_client.cli.connectors import connectors
 from gretel_client.cli.errors import ExceptionHandler
+from gretel_client.cli.hybrid import hybrid
 from gretel_client.cli.models import models
 from gretel_client.cli.projects import projects
 from gretel_client.cli.records import records
@@ -29,8 +29,9 @@ class GretelCliHandler(ExceptionHandler):
 @click.option("--debug/--no-debug", default=False, help="Show extra debug messages.")
 @click.option(
     "--output",
-    type=click.Choice(["json"], case_sensitive=False),
-    default="json",
+    type=click.Choice(["auto", "json"], case_sensitive=False),
+    default="auto",
+    help="Control how output data is formatted.",
 )
 @click.pass_context
 def cli(ctx: click.Context, debug: bool, output: str):
@@ -53,12 +54,20 @@ def _check_endpoint(endpoint: str) -> str:
     metavar="URL",
     help="Gretel API endpoint.",
 )
+@click.option(
+    "--artifact-endpoint",
+    prompt="Artifact Endpoint",
+    default=lambda: get_session_config().artifact_endpoint,
+    metavar="URL",
+    help="Specify the endpoint for project and model artifacts.",
+)
 @click.option(
     "--default-runner",
     prompt="Default Runner",
     default=lambda: get_session_config().default_runner,
     type=click.Choice(
-        [RunnerMode.CLOUD.value, RunnerMode.LOCAL.value], case_sensitive=False
+        [RunnerMode.CLOUD.value, RunnerMode.LOCAL.value, RunnerMode.HYBRID.value],
+        case_sensitive=False,
     ),
     metavar="RUNNER",
     help="Specify the default runner.",
@@ -80,16 +89,24 @@ def _check_endpoint(endpoint: str) -> str:
 )
 @pass_session
 def configure(
-    sc: SessionContext, endpoint: str, api_key: str, project: str, default_runner: str
+    sc: SessionContext,
+    endpoint: str,
+    artifact_endpoint: str,
+    api_key: str,
+    project: str,
+    default_runner: str,
 ):
     project_name = None if project == "none" else project
     endpoint = _check_endpoint(endpoint)
     # swap the api key back to the original if one was
     # already read in from the config and not updated
     if api_key.endswith("****"):
-        api_key = get_session_config().api_key
+        api_key = sc.session.api_key
     config = ClientConfig(
-        endpoint=endpoint, api_key=api_key, default_runner=default_runner
+        endpoint=endpoint,
+        artifact_endpoint=artifact_endpoint,
+        api_key=api_key,
+        default_runner=default_runner,
     )
     config.update_default_project(project_id=project_name)
     configure_session(config)
@@ -103,7 +120,7 @@ def configure(
 @cli.command(help="Check account, user and configuration details.")
 @pass_session
 def whoami(sc: SessionContext):
-    me = users.get_me()
+    me = users.get_me(session=sc.session)
     sc.print(data={f.EMAIL: me[f.EMAIL], "config": sc.config.masked})
 
 
@@ -112,8 +129,8 @@ cli.add_command(records)
 cli.add_command(projects)
 cli.add_command(artifacts)
 cli.add_command(connections)
-cli.add_command(connectors)
 cli.add_command(workflows)
+cli.add_command(hybrid)
 cli.add_command(agent)
 
 
