@@ -8,7 +8,6 @@ import click
 import yaml
 
 from gretel_client.cli.common import (
-    get_model,
     model_option,
     pass_session,
     poll_and_print,
@@ -25,6 +24,7 @@ from gretel_client.config import RunnerMode
 from gretel_client.models.config import get_model_type_config, GPU
 from gretel_client.projects.docker import ContainerRun, ContainerRunError
 from gretel_client.projects.jobs import Status
+from gretel_client.projects.models import Model
 from gretel_client.projects.records import RecordHandler
 
 LOCAL = "__local__"
@@ -44,14 +44,9 @@ def model_path_option(fn):
 
 
 def input_data_option(fn):
-    def callback(ctx, param: click.Option, value: str):
-        gc: SessionContext = ctx.ensure_object(SessionContext)
-        return value or gc.data_source
-
     return click.option(  # type: ignore
         "--in-data",
         metavar="PATH",
-        callback=callback,
         help="Specify the model input data.",
     )(fn)
 
@@ -90,26 +85,15 @@ def _validate_params(
             "--model-path", "Cannot specify the local model path for cloud models."
         )
 
-    if runner == RunnerMode.MANUAL.value and (output or model_path):
+    if runner in (RunnerMode.MANUAL.value, RunnerMode.HYBRID.value) and (
+        output or model_path
+    ):
         raise click.BadOptionUsage(
             "--runner",
-            "--runner manual cannot be used together with any of --output, --model-path.",
+            f"--runner {runner} cannot be used together with any of --output, --model-path.",
         )
 
 
-def _check_model_and_runner(sc: SessionContext, runner) -> str:
-    if not sc.model.is_cloud_model and runner is RunnerMode.CLOUD.value:
-        sc.log.info(
-            (
-                f"The model {sc.model.model_id} is configured for local runs, "
-                f"but runner_mode is {runner}."
-            )
-        )
-        sc.log.info("Setting runner_mode to local and running the model.")
-        return RunnerMode.LOCAL.value
-    return runner
-
-
 def _configure_data_source(
     sc: SessionContext, in_data: Optional[str], runner: str
 ) -> Optional[str]:
@@ -137,19 +121,10 @@ def _configure_data_source(
     return data_source
 
 
-def _configure_ref_data(sc: SessionContext, ref_data: RefData, runner: str) -> RefData:
-    if not ref_data.is_empty and runner == RunnerMode.MANUAL.value:
-        return ref_data
-    if not ref_data.is_empty and runner == RunnerMode.CLOUD.value:
-        sc.log.info("Uploading ref data...")
-        ref_dict = {}
-        for key, data_source in ref_data.ref_dict.items():
-            data_source = sc.project.upload_artifact(data_source)
-            ref_dict[key] = data_source
-        ref_data = ref_data_factory(ref_dict)
-    else:
-        # If the job is being run locally, we swap the data sources to __local__
-        # so we don't expose anything in the stored cloud config
+def _configure_ref_data(ref_data: RefData, runner: str) -> RefData:
+    # If the job is being run locally, we swap the data sources to __local__
+    # so we don't expose anything in the stored cloud config
+    if runner == RunnerMode.LOCAL.value:
         for key, data_source in ref_data.ref_dict.items():
             ref_data.ref_dict[key] = LOCAL
     return ref_data
@@ -159,7 +134,6 @@ def create_and_run_record_handler(
     sc: SessionContext,
     *,
     params: Optional[dict],
-    action: Optional[str],
     runner: str,
     output: str,
     in_data: Optional[str],
@@ -167,7 +141,6 @@ def create_and_run_record_handler(
     data_source: Optional[str],
     status_strings: StatusDescriptions,
     ref_data: Optional[RefData] = None,
-    config_ref_data: Optional[RefData] = None,
 ):
 
     # NOTE: ``in_data`` is only evaluated to determine if we should set a --data-source
@@ -175,19 +148,13 @@ def create_and_run_record_handler(
     # is what will be sent to the Cloud API.
 
     sc.log.info(f"Creating record handler for model {sc.model.model_id}.")
-    record_handler = sc.model.create_record_handler_obj()
-
-    if config_ref_data is None:
-        config_ref_data = ref_data_factory()
-
-    data = record_handler._submit(
+    record_handler = sc.model.create_record_handler_obj(
         params=params,
-        action=action,
-        runner_mode=RunnerMode(runner),
         data_source=data_source,
-        ref_data=config_ref_data,
-        _default_manual=True,
+        ref_data=ref_data,
     )
+
+    data = record_handler.submit(runner_mode=RunnerMode.parse(runner))
     sc.register_cleanup(lambda: record_handler.cancel())
     sc.log.info(f"Record handler created {record_handler.record_id}.")
 
@@ -283,11 +250,11 @@ def generate(
     output: str,
     in_data: str,
     model_path: str,
-    model_id: str,
+    model_id: dict,
     num_records: int,
     max_invalid: int,
 ):
-    runner = _check_model_and_runner(sc, runner)
+    runner = sc.runner
     _validate_params(sc, runner, output, model_path, in_data)
 
     data_source = _configure_data_source(sc, in_data, runner)
@@ -301,7 +268,6 @@ def generate(
         sc,
         params=params,
         data_source=data_source,
-        action="generate",
         runner=runner,
         output=output,
         in_data=in_data,
@@ -327,7 +293,7 @@ def transform(
     runner: str,
     model_id: str,
 ):
-    runner = _check_model_and_runner(sc, runner)
+    runner = sc.runner
     _validate_params(sc, runner, output, model_path, in_data)
 
     data_source = _configure_data_source(sc, in_data, runner)
@@ -336,7 +302,6 @@ def transform(
         sc,
         params=None,
         data_source=data_source,
-        action="transform",
         runner=runner,
         output=output,
         in_data=in_data,
@@ -362,7 +327,7 @@ def classify(
     model_id: str,
     model_path: str,
 ):
-    runner = _check_model_and_runner(sc, runner)
+    runner = sc.runner
     _validate_params(sc, runner, output, model_path, in_data)
 
     data_source = _configure_data_source(sc, in_data, runner)
@@ -371,7 +336,6 @@ def classify(
         sc,
         params=None,
         data_source=data_source,
-        action="classify",
         runner=runner,
         output=output,
         in_data=in_data,
@@ -380,10 +344,6 @@ def classify(
     )
 
 
-def action_option(fn):
-    return click.option("--action", help="Specify action to run.", type=str)(fn)
-
-
 @models.command(help="Run an existing model.")
 @project_option
 @runner_option
@@ -393,7 +353,6 @@ def action_option(fn):
 @output_data_option
 @model_option
 @pass_session
-@action_option
 @click.option(
     "--param",
     type=(str, str),
@@ -402,7 +361,9 @@ def action_option(fn):
 )
 @click.option(
     "--params-file",
-    type=Path,
+    type=click.Path(
+        exists=True, file_okay=True, dir_okay=False, readable=True, path_type=Path
+    ),
     help="Specify a file (YAML or JSON) with all parameters to pass into the record handler.",
 )
 def run(
@@ -410,12 +371,11 @@ def run(
     project: str,
     model_id: str,
     in_data: Optional[str],
-    ref_data: Tuple[str],
+    ref_data: tuple[str, ...],
     output: str,
     runner: str,
     model_path: str,
-    action: Optional[str],
-    param: List[Tuple[str, str]],
+    param: tuple[tuple[str, str], ...],
     params_file: Optional[Path],
 ):
     """
@@ -434,7 +394,7 @@ def run(
             )
         extra_params = {key: value for key, value in param}
 
-    runner = _check_model_and_runner(sc, runner)
+    runner = sc.runner
     _validate_params(sc, runner, output, model_path, None)
 
     # The idea here:
@@ -444,20 +404,15 @@ def run(
     if in_data:
         data_source = _configure_data_source(sc, in_data, runner)
 
-    # The idea here:
-    # - ref_data is what the CLI arguments were
-    # - config_ref_data is what is going to be sent to the API in the model config
     ref_data = ref_data_factory(ref_data)
-    config_ref_data = _configure_ref_data(sc, deepcopy(ref_data), runner)
+    ref_data = _configure_ref_data(ref_data, runner)
 
     create_and_run_record_handler(
         sc,
         params=extra_params,
-        action=action,
         in_data=in_data,
         data_source=data_source,
         ref_data=ref_data,
-        config_ref_data=config_ref_data,
         runner=runner,
         output=output,
         status_strings=get_model_type_config().run_status_descriptions,
@@ -478,20 +433,29 @@ def run(
     metavar="ID",
     help="Specify the model.",
     required=False,
-    callback=get_model,
-    default="None",
 )
 @record_handler_option
 @pass_session
 def get(
-    sc: SessionContext, record_handler_id: str, model_id: str, project: str, output: str
+    sc: SessionContext,
+    record_handler_id: dict,
+    model_id: Optional[str],
+    project: str,
+    output: str,
 ):
-    if model_id == "None":
+    rh_model_id = record_handler_id.get("model_id")
+    if not rh_model_id and not model_id:
+        raise click.BadOptionUsage("--model-id", "Please specify a model ID")
+    if rh_model_id and model_id and rh_model_id != model_id:
         raise click.BadOptionUsage(
-            "--model-id", "Please specify the option '--model-id'."
+            "--model-id",
+            "Explicitly specified model ID does not match model ID found in record handler JSON file",
         )
+
+    model_id = model_id or rh_model_id
+
     record_handler: RecordHandler = sc.project.get_model(model_id).get_record_handler(
-        record_handler_id
+        record_handler_id["uid"]
     )
     if record_handler.status != "completed":
         sc.log.error(
