@@ -20,11 +20,6 @@ def get_status_description(
 
 
 class ModelTypeConfig(ABC):
-    @property
-    @abstractmethod
-    def action_name(self) -> Optional[str]:
-        ...
-
     @property
     def train_instance_type(self) -> str:
         return CPU
@@ -67,10 +62,6 @@ class ModelTypeConfig(ABC):
 
 
 class GenericModelTypeConfig(ModelTypeConfig):
-    @property
-    def action_name(self) -> Optional[str]:
-        return None
-
     @property
     def run_status_descriptions(self) -> StatusDescriptions:
         return {
@@ -92,10 +83,6 @@ class GenericModelTypeConfig(ModelTypeConfig):
 
 
 class TransformModelTypeConfig(ModelTypeConfig):
-    @property
-    def action_name(self) -> Optional[str]:
-        return "transform"
-
     @property
     def run_status_descriptions(self) -> StatusDescriptions:
         return {
@@ -124,10 +111,6 @@ class TransformModelTypeConfig(ModelTypeConfig):
 
 
 class ClassifyModelTypeConfig(ModelTypeConfig):
-    @property
-    def action_name(self) -> Optional[str]:
-        return "classify"
-
     @property
     def run_status_descriptions(self) -> StatusDescriptions:
         return {
@@ -150,10 +133,6 @@ class ClassifyModelTypeConfig(ModelTypeConfig):
 
 
 class SyntheticsModelTypeConfig(ModelTypeConfig):
-    @property
-    def action_name(self) -> Optional[str]:
-        return "generate"
-
     @property
     def train_instance_type(self) -> str:
         return GPU
@@ -179,7 +158,7 @@ class SyntheticsModelTypeConfig(ModelTypeConfig):
         }
 
     def peek_report(self, report_contents: dict) -> Optional[dict]:
-        return _peek_sqs_report(report_contents)
+        return _peek_any_report(report_contents)
 
 
 class CtganModelTypeConfig(GenericModelTypeConfig):
@@ -192,7 +171,7 @@ class CtganModelTypeConfig(GenericModelTypeConfig):
         return GPU
 
     def peek_report(self, report_contents: dict) -> Optional[dict]:
-        return _peek_sqs_report(report_contents)
+        return _peek_any_report(report_contents)
 
 
 class ActganModelTypeConfig(GenericModelTypeConfig):
@@ -205,7 +184,7 @@ class ActganModelTypeConfig(GenericModelTypeConfig):
         return GPU
 
     def peek_report(self, report_contents: dict) -> Optional[dict]:
-        return _peek_sqs_report(report_contents)
+        return _peek_any_report(report_contents)
 
 
 class GptXModelTypeConfig(GenericModelTypeConfig):
@@ -213,16 +192,19 @@ class GptXModelTypeConfig(GenericModelTypeConfig):
     def train_instance_type(self) -> str:
         return GPU
 
+    def peek_report(self, report_contents: dict) -> Optional[dict]:
+        return _peek_any_report(report_contents)
+
 
 class AmplifyModelTypeConfig(GenericModelTypeConfig):
     @property
     def peek_report(self, report_contents: dict) -> str:
-        return _peek_sqs_report(report_contents)
+        return _peek_any_report(report_contents)
 
 
 class EvaluateModelTypeConfig(GenericModelTypeConfig):
     def peek_report(self, report_contents: dict) -> Optional[dict]:
-        return _peek_sqs_report(report_contents)
+        return _peek_any_report(report_contents)
 
 
 class TimeseriesDganModelTypeConfig(GenericModelTypeConfig):
@@ -237,13 +219,19 @@ class TimeseriesDganModelTypeConfig(GenericModelTypeConfig):
 
 
 
-def _peek_sqs_report(report_contents) -> dict:
+def _peek_any_report(report_contents) -> dict:
+    """
+    Extract the fields from any synthetic report (SQS, MQS, Text SQS).
+    """
     fields = [
         "synthetic_data_quality_score",
         "field_correlation_stability",
         "principal_component_stability",
         "field_distribution_stability",
         "privacy_protection_level",
+        "average_metric_difference",
+        "semantic_similarity",
+        "structure_similarity",
     ]
     return {f: report_contents[f] for f in fields if f in report_contents}
 
