@@ -5,21 +5,37 @@ import json
 import time
 
 from abc import ABC, abstractmethod, abstractproperty
+from contextlib import contextmanager
 from dataclasses import dataclass, field
 from enum import Enum
 from pathlib import Path
-from typing import Callable, Iterator, List, Optional, Tuple, Type, TYPE_CHECKING, Union
-from urllib.parse import urlparse
+from typing import (
+    BinaryIO,
+    Callable,
+    Iterator,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    TYPE_CHECKING,
+    Union,
+)
 
-import requests
 import smart_open
 
 import gretel_client.rest.exceptions
 
 from gretel_client.cli.utils.parser_utils import RefData
-from gretel_client.config import DEFAULT_RUNNER, get_logger, RunnerMode
+from gretel_client.config import ClientConfig, get_logger, RunnerMode
+from gretel_client.dataframe import _DataFrameT
 from gretel_client.models.config import get_model_type_config
-from gretel_client.projects.common import _DataFrameT, f, ModelArtifact, WAIT_UNTIL_DONE
+from gretel_client.projects.artifact_handlers import (
+    ArtifactsHandler,
+    CloudArtifactsHandler,
+    get_transport_params,
+    HybridArtifactsHandler,
+)
+from gretel_client.projects.common import f, ModelArtifact, WAIT_UNTIL_DONE
 from gretel_client.projects.exceptions import GretelJobNotFound, WaitTimeExceeded
 from gretel_client.rest.api.projects_api import ProjectsApi
 
@@ -83,21 +99,42 @@ class Job(ABC):
             self._poll_job_endpoint()
         self._logs_iter_index = 0
 
+    @property
+    def session(self) -> ClientConfig:
+        return self.project.session
+
     def submit(
-        self, runner_mode: Union[str, RunnerMode] = DEFAULT_RUNNER, **kwargs
-    ) -> dict:
-        # todo: deprecate in favor of submit_manual and submit_cloud
-        # Support `runner_mode` input as a str to provide
-        # parity with the CLI semantics
-        if isinstance(runner_mode, str):
-            try:
-                runner_mode = RunnerMode(runner_mode)
-            except ValueError:
-                raise ValueError(f"Invalid runner_mode: {runner_mode}")
+        self,
+        runner_mode: Optional[Union[str, RunnerMode]] = None,
+        dry_run: bool = False,
+    ) -> Job:
+        """Submit this Job to the Gretel Cloud API.
 
-        return self._submit(runner_mode, **kwargs).print_obj
+        Args:
+            runner_mode: Determines where to run the model. If not specified, the
+                runner mode of the project (if configured) is used, otherwise
+                the default runner mode of the session is used.
+            dry_run: If set to True the model config will be submitted for
+                validation, but won't be run. Ignored for record handlers.
+        """
+        if runner_mode is None:
+            runner_mode = self.project.runner_mode or self.session.default_runner
+        runner_mode = RunnerMode.parse(runner_mode)
+        if self.project.runner_mode and runner_mode != self.project.runner_mode:
+            raise ValueError(
+                f"Specified runner mode '{runner_mode.value}' is different from project runner mode '{self.project.runner_mode.value}'"
+            )
 
-    def submit_manual(self) -> Job:
+        if runner_mode == RunnerMode.CLOUD:
+            return self.submit_cloud(dry_run)
+        elif runner_mode == RunnerMode.HYBRID:
+            return self.submit_hybrid(dry_run)
+        elif runner_mode == RunnerMode.MANUAL:
+            return self.submit_manual(dry_run)
+        elif runner_mode == RunnerMode.LOCAL:
+            return self.submit_local(dry_run)
+
+    def submit_manual(self, dry_run: bool = False) -> Job:
         """Submit this Job to the Gretel Cloud API, which will create
         the job metadata but no runner will be started. The ``Model`` instance
         can now be passed into a dedicated runner.
@@ -105,25 +142,57 @@ class Job(ABC):
         Returns:
             The response from the Gretel API.
         """
-        return self._submit(runner_mode=RunnerMode.MANUAL)
+        return self._submit(runner_mode=RunnerMode.MANUAL, dry_run=dry_run)
 
-    def submit_cloud(self) -> Job:
-        """Submit this model to be scheduled for runing in Gretel Cloud.
+    def submit_local(self, dry_run: bool = False) -> Job:
+        """Submit this Job to the Gretel Cloud API to be scheduled for running in a local container.
 
         Returns:
             The response from the Gretel API.
         """
+        return self._submit(runner_mode=RunnerMode.LOCAL, dry_run=dry_run)
+
+    def submit_cloud(self, dry_run: bool = False) -> Job:
+        """Submit this Job to the Gretel Cloud API be scheduled for running in Gretel Cloud.
+
+        Returns:
+            The response from the Gretel API.
+        """
+        return self._submit_remote(
+            runner_mode=RunnerMode.CLOUD,
+            artifacts_handler=self.project.cloud_artifacts_handler,
+            dry_run=dry_run,
+        )
+
+    def submit_hybrid(self, dry_run: bool = False) -> Job:
+        """Submit this Job to the Gretel Cloud API to be scheduled for running in a hybrid deployment.
+
+        Returns:
+            The response from the Gretel API.
+        """
+        return self._submit_remote(
+            runner_mode=RunnerMode.HYBRID,
+            artifacts_handler=self.project.hybrid_artifacts_handler,
+            dry_run=dry_run,
+        )
+
+    def _submit_remote(
+        self,
+        runner_mode: RunnerMode,
+        artifacts_handler: Union[CloudArtifactsHandler, HybridArtifactsHandler],
+        dry_run: bool,
+    ) -> Job:
         if (
             isinstance(self.data_source, _DataFrameT)
             and not self.data_source.empty
             or self.data_source
         ):
-            self.upload_data_source()
+            self.upload_data_source(_artifacts_handler=artifacts_handler)
 
         if not self.ref_data.is_empty:
-            self.upload_ref_data()
+            self.upload_ref_data(_artifacts_handler=artifacts_handler)
 
-        return self._submit(runner_mode=RunnerMode.CLOUD)
+        return self._submit(runner_mode=runner_mode, dry_run=dry_run)
 
     @abstractmethod
     def _submit(self, runner_mode: RunnerMode, **kwargs) -> Job:
@@ -134,7 +203,7 @@ class Job(ABC):
         ...
 
     @abstractmethod
-    def _do_get_job_details(self) -> dict:
+    def _do_get_job_details(self, extra_expand: Optional[list[str]] = None) -> dict:
         ...
 
     @abstractmethod
@@ -180,7 +249,7 @@ class Job(ABC):
 
     @property
     def runner_mode(self) -> str:
-        """Returns the runner_mode of the job. May be one of ``manual`` or ``cloud``."""
+        """Returns the runner_mode of the job. May be one of ``hybrid``, ``manual`` or ``cloud``."""
         return self._data[self.job_type][f.RUNNER_MODE]
 
     @property
@@ -221,7 +290,11 @@ class Job(ABC):
 
     # Base Job Methods
 
-    def upload_data_source(self, _validate: bool = True) -> Optional[str]:
+    def upload_data_source(
+        self,
+        _validate: bool = True,
+        _artifacts_handler: Union[CloudArtifactsHandler, HybridArtifactsHandler] = None,
+    ) -> Optional[str]:
         """Resolves and uploads the data source specified in the
         model config.
 
@@ -236,10 +309,18 @@ class Job(ABC):
             or self.data_source
         ):
             # NOTE: This assignment re-writes the gretel artifact onto the config
-            self.data_source = self.project.upload_artifact(self.data_source, _validate)
+            self.data_source = self.project.upload_artifact(
+                artifact_path=self.data_source,
+                _validate=_validate,
+                _artifacts_handler=_artifacts_handler,
+            )
             return self.data_source
 
-    def upload_ref_data(self, _validate: bool = True) -> RefData:
+    def upload_ref_data(
+        self,
+        _validate: bool = True,
+        _artifacts_handler: Optional[ArtifactsHandler] = None,
+    ) -> RefData:
         """
         Resolves and uploads ref data sources specificed in the model config.
 
@@ -256,7 +337,11 @@ class Job(ABC):
         # Loop over each data source and try and upload to Gretel
         ref_data_dict = curr_ref_data.ref_dict
         for key, data_source in ref_data_dict.items():
-            gretel_key = self.project.upload_artifact(data_source, _validate)
+            gretel_key = self.project.upload_artifact(
+                artifact_path=data_source,
+                _validate=_validate,
+                _artifacts_handler=_artifacts_handler,
+            )
             ref_data_dict[key] = gretel_key
 
         new_ref_data = RefData(ref_data_dict)
@@ -271,12 +356,19 @@ class Job(ABC):
         for artifact_type in self.artifact_types:
             yield artifact_type, self.get_artifact_link(artifact_type)
 
+    def get_artifacts_by_artifact_types(
+        self, artifact_types: List[str]
+    ) -> Iterator[Tuple[str, str]]:
+        """List artifact links for all known artifact types."""
+        for artifact_type in artifact_types:
+            yield artifact_type, self.get_artifact_link(artifact_type)
+
     def get_artifact_link(self, artifact_key: str) -> str:
         """Retrieves a signed S3 link that will download the specified
         artifact type.
 
         Args:
-            artifact_type: Artifact type to download.
+            artifact_key: Artifact type to download.
         """
         if artifact_key not in self.artifact_types:
             raise Exception(
@@ -284,6 +376,25 @@ class Job(ABC):
             )
         return self._do_get_artifact(artifact_key)
 
+    @contextmanager
+    def get_artifact_handle(self, artifact_key: str) -> BinaryIO:
+        """Returns a reference to a remote artifact that can be used to
+        read binary data within a context manager
+
+        >>> with job.get_artifact_handle("report_json") as file:
+        ...   print(file.read())
+
+        Args:
+            artifact_key: Artifact type to download.
+
+        Returns:
+            a file like object
+        """
+        link = self.get_artifact_link(artifact_key)
+        transport_params = get_transport_params(link)
+        with smart_open.open(link, "rb", transport_params=transport_params) as handle:
+            yield handle
+
     def download_artifacts(self, target_dir: Union[str, Path]):
         """Given a target directory, either as a string or a Path object, attempt to enumerate
         and download all artifacts associated with this Job
@@ -296,24 +407,83 @@ class Job(ABC):
         output_path = Path(target_dir)
         output_path.mkdir(exist_ok=True, parents=True)
         log.info(f"Downloading model artifacts to {output_path.resolve()}")
-        for artifact_type, download_link in self.get_artifacts():
+        for artifact_type, download_link in self._get_artifacts_to_download():
             # we don't need to download cloud model artifacts
             if artifact_type == ModelArtifact.MODEL.value:
                 continue
+            self.project.default_artifacts_handler.download(
+                download_link, output_path, artifact_type, log
+            )
+
+    def _get_artifacts_to_download(self) -> Iterator[Tuple[str, str]]:
+        if self.runner_mode == "cloud":
+            resp = self._do_get_job_details([f.ARTIFACTS])
+            artifact_types = [
+                a.get("name") for a in resp.get(f.DATA).get(f.ARTIFACTS, [])
+            ]
+            return self.get_artifacts_by_artifact_types(artifact_types)
+        return self.get_artifacts()
+
+    def _get_report_contents(
+        self, report_path: Optional[str] = None, artifact_type: Optional[str] = None
+    ) -> Optional[dict]:
+        """
+        Helper to encapsulate try/except boilerplate for peek_report and get_report_summary.
+        """
+        if report_path is None and artifact_type is not None:
             try:
-                art = requests.get(download_link)
-                if art.status_code == 200:
-                    art_output_path = (
-                        output_path / Path(urlparse(download_link).path).name
+                report_path = self.get_artifact_link(artifact_type)
+            except Exception:
+                pass
+
+        report_contents = None
+        if report_path:
+            try:
+                transport_params = get_transport_params(report_path)
+                with smart_open.open(
+                    report_path, "rb", transport_params=transport_params
+                ) as rh:  # type:ignore
+                    report_contents = rh.read()
+            except Exception:
+                pass
+
+        if report_contents:
+            try:
+                return json.loads(report_contents)
+            except Exception:
+                pass
+
+    def _get_report_contents_wrapper(
+        self, report_path: Optional[str] = None
+    ) -> Optional[dict]:
+        report_contents = None
+        if report_path:
+            report_contents = self._get_report_contents(report_path=report_path)
+        else:
+            try:
+                if self.model_type == "gpt_x":
+                    report_contents = self._get_report_contents(
+                        artifact_type="text_metrics_report_json"
                     )
-                    with open(art_output_path, "wb+") as out:
-                        log.info(f"\tWriting {artifact_type} to {art_output_path}")
-                        out.write(art.content)
-            except requests.exceptions.HTTPError as ex:
-                log.error(
-                    f"\tCould not download {artifact_type}. You might retry this request.",
-                    ex=ex,
-                )
+                elif self.model_type == "evaluate":
+                    for artifact_type in [
+                        "report_json",
+                        "classification_report_json",
+                        "regression_report_json",
+                        "text_metrics_report_json",
+                    ]:
+                        report_contents = self._get_report_contents(
+                            artifact_type=artifact_type
+                        )
+                        if report_contents:
+                            break
+                else:
+                    report_contents = self._get_report_contents(
+                        artifact_type="report_json"
+                    )
+            except Exception:
+                pass
+        return report_contents
 
     def _peek_report(self, report_contents: dict) -> Optional[dict]:
         return get_model_type_config(self.model_type).peek_report(report_contents)
@@ -327,23 +497,10 @@ class Job(ABC):
                 is passed, the function will check for a cloud
                 based artifact.
         """
-        if report_path is None:
-            try:
-                report_path = self.get_artifact_link("report_json")
-            except Exception:
-                pass
-
-        report_contents = None
-        if report_path:
-            try:
-                with smart_open.open(report_path, "rb") as rh:  # type:ignore
-                    report_contents = rh.read()
-            except Exception:
-                pass
+        report_contents = self._get_report_contents_wrapper(report_path=report_path)
 
         if report_contents:
             try:
-                report_contents = json.loads(report_contents)
                 return self._peek_report(report_contents)
             except Exception:
                 pass
@@ -361,21 +518,10 @@ class Job(ABC):
                 is passed, the function will check for a cloud
                 report artifact.
         """
-        if not report_path:
-            try:
-                report_path = self.get_artifact_link("report_json")
-            except Exception:
-                pass
-        report_contents = None
-        if report_path:
-            try:
-                with smart_open.open(report_path, "rb") as rh:  # type:ignore
-                    report_contents = rh.read()
-            except Exception:
-                pass
+        report_contents = self._get_report_contents_wrapper(report_path=report_path)
+
         if report_contents:
             try:
-                report_contents = json.loads(report_contents)
                 return self._get_report_summary(report_contents)
             except Exception:
                 pass
