@@ -1,33 +1,40 @@
 """
 High level API for interacting with a Gretel Project
 """
-import uuid
+from __future__ import annotations
 
 from contextlib import contextmanager
 from functools import wraps
-from io import BytesIO
 from pathlib import Path
-from typing import Any, Dict, Iterator, List, Optional, Type, TypeVar, Union
-from urllib.parse import urlparse
+from typing import Any, BinaryIO, Dict, Iterator, List, Optional, Type, TypeVar, Union
 
-import requests
-import smart_open
-
-from tenacity import retry, retry_if_exception_type, stop_after_attempt, wait_fixed
+from backports.cached_property import cached_property
 
 from gretel_client.cli.utils.parser_utils import (
     DataSourceTypes,
     ref_data_factory,
     RefDataTypes,
 )
-from gretel_client.config import get_logger, get_session_config
-from gretel_client.projects.common import _DataFrameT, f, validate_data_source
+from gretel_client.config import (
+    ClientConfig,
+    get_logger,
+    get_session_config,
+    RunnerMode,
+)
+from gretel_client.dataframe import _DataFrameT
+from gretel_client.projects.artifact_handlers import (
+    ArtifactsHandler,
+    cloud_handler,
+    CloudArtifactsHandler,
+    hybrid_handler,
+    HybridArtifactsHandler,
+)
+from gretel_client.projects.common import f
 from gretel_client.projects.exceptions import GretelProjectError
 from gretel_client.projects.models import Model
 from gretel_client.rest import models
 from gretel_client.rest.api.projects_api import ProjectsApi
-from gretel_client.rest.exceptions import NotFoundException, UnauthorizedException
-from gretel_client.rest.model.artifact import Artifact
+from gretel_client.rest.exceptions import ForbiddenException, UnauthorizedException
 from gretel_client.users.users import get_me
 
 DATA = "data"
@@ -36,7 +43,7 @@ PROJECT = "project"
 MODELS = "models"
 
 
-log = get_logger(__name__)
+logger = get_logger(__name__)
 
 
 def check_not_deleted(func):
@@ -54,23 +61,11 @@ def check_not_deleted(func):
 MT = TypeVar("MT", dict, Model)
 
 
-# These exceptions are used for control flow with retries in get_artifact_manifest.
-# They are NOT intended to bubble up out of this module.
-class ManifestNotFoundException(Exception):
-    pass
-
-
-class ManifestPendingException(Exception):
-    def __init__(self, msg: Optional[str] = None, manifest: dict = {}):
-        # Piggyback the pending manifest onto the exception.  If we give up, we still want to pass it back as a normal return value.
-        self.manifest = manifest
-        super().__init__(msg)
-
-
 class Project:
-    """Represents Gretel project. In general you should not have
-    to init this class directly, but can make use of the factory
-    method from ``get_project``.
+    """Represents a Gretel project. In general you should not init
+    this class directly, but always make use of the factory methods
+    ``get_project``, ``create_project``, ``get_or_create_project``
+    etc.
 
     Args:
         name: The unique name of the project. This is either set by you or auto
@@ -79,8 +74,12 @@ class Project:
             gretel and never changes.
         desc: A short description of the project
         display_name: The main display name used in the Gretel Console for your project
+        session: The client session to use for API interactions, or ``None`` to use the
+            default session.
     """
 
+    runner_mode: Optional[RunnerMode]
+
     def __init__(
         self,
         *,
@@ -89,16 +88,69 @@ class Project:
         project_guid: Optional[str] = None,
         desc: Optional[str] = None,
         display_name: Optional[str] = None,
+        runner_mode: Optional[Union[RunnerMode, str]] = None,
+        session: Optional[ClientConfig] = None,
     ):
-        self.client_config = get_session_config()
+        self.client_config = session or get_session_config()
         self.projects_api = self.client_config.get_api(ProjectsApi)
         self.name = name
         self.project_id = project_id
         self.project_guid = project_guid
         self.description = desc
         self.display_name = display_name
+        self.runner_mode = RunnerMode.parse_optional(runner_mode)
         self._deleted = False
 
+    @property
+    def session(self) -> Optional[ClientConfig]:
+        return self.client_config
+
+    @check_not_deleted
+    def with_session(self, session: ClientConfig) -> Project:
+        if self.session is session:
+            return self
+        if (
+            session.endpoint != self.session.endpoint
+            or session.api_key != self.session.api_key
+        ):
+            raise ValueError(
+                "new session must match API endpoint and API key of existing one"
+            )
+
+        return Project(
+            name=self.name,
+            project_id=self.project_id,
+            project_guid=self.project_guid,
+            desc=self.description,
+            display_name=self.display_name,
+            runner_mode=self.runner_mode,
+            session=session,
+        )
+
+    @cached_property
+    def default_artifacts_handler(self) -> ArtifactsHandler:
+        runner = self.runner_mode or RunnerMode.parse_optional(
+            self.client_config.default_runner
+        )
+
+        if runner == RunnerMode.HYBRID:
+            return self.hybrid_artifacts_handler
+        elif runner == RunnerMode.CLOUD:
+            return self.cloud_artifacts_handler
+        else:
+            raise GretelProjectError(
+                f"Artifact handling is not supported under {runner.value} runner mode. "
+                "Please update the default runner in your configuration to `cloud` or `hybrid` to work with artifacts."
+            )
+
+    @cached_property
+    def cloud_artifacts_handler(self) -> CloudArtifactsHandler:
+        return cloud_handler(self)
+
+    @cached_property
+    def hybrid_artifacts_handler(self) -> HybridArtifactsHandler:
+        return hybrid_handler(self)
+
     @check_not_deleted
     def delete(self, *args, **kwargs):
         """Deletes a project. After the project has been deleted, functions
@@ -109,7 +161,7 @@ class Project:
         """
         # todo(dn): remove in 0.9 release
         if kwargs.get("include_models") or len(args) > 0:
-            log.warning(
+            logger.warning(
                 "``include_models`` is deprecated and will be removed in the 0.9 release."
             )
         self.projects_api.delete_project(project_id=self.project_id)
@@ -118,19 +170,25 @@ class Project:
     @check_not_deleted
     def get_console_url(self) -> str:
         """Returns web link to access the project from Gretel's console."""
-        console_base = self.client_config.endpoint.replace("api", "console")
+        console_base = self.client_config.endpoint.replace("api", "console").replace(
+            "cloud", "ai"
+        )
         return f"{console_base}/{self.project_guid}"
 
     @property
     def as_dict(self) -> dict:
         """Returns a dictionary representation of the project."""
-        return {
+        dict_data = {
             "name": self.name,
             "project_id": self.project_id,
+            "project_guid": self.project_guid,
             "display_name": self.display_name,
             "desc": self.description,
             "console_url": self.get_console_url(),
         }
+        if self.runner_mode is not None:
+            dict_data["runner_mode"] = self.runner_mode.value
+        return dict_data
 
     @check_not_deleted
     def info(self) -> dict:
@@ -139,25 +197,32 @@ class Project:
 
     @check_not_deleted
     def search_models(
-        self, factory: Type[MT] = Model, limit: int = 100
+        self,
+        factory: Type[MT] = Model,
+        limit: int = 100,
+        model_name: str = "",
     ) -> Iterator[MT]:
         """Search for project models.
 
         Args:
-            limit: Limits the number of project models to return
             factory: Determines what type of Model representation is returned.
                 If ``Model`` is passed, a ``Model`` will be returned. If ``dict``
                 is passed, a dictionary representation of the search results
                 will be returned.
+            limit: Limits the number of project models to return
+            model_name: Name of the model to try and match on (partial match)
         """
         if factory not in (dict, Model):
-            raise ValueError("factory must be one of ``str`` or ``Model``.")
-        models = (
-            self.projects_api.get_models(project_id=self.name, limit=limit)
-            .get(DATA)
-            .get(MODELS)
-        )
-        for model in models:
+            raise ValueError("factory must be one of ``dict`` or ``Model``.")
+
+        api_args = {"project_id": self.name, "limit": limit}
+        if model_name:
+            api_args["model_name"] = model_name
+
+        result = self.projects_api.get_models(**api_args)
+        searched_models = result.get(DATA).get(MODELS)
+
+        for model in searched_models:
             if factory == Model:
                 model = self.get_model(model_id=model[f.UID])
             yield model
@@ -209,14 +274,13 @@ class Project:
     @check_not_deleted
     def artifacts(self) -> List[dict]:
         """Returns a list of project artifacts."""
-        return (
-            self.projects_api.get_artifacts(project_id=self.name)
-            .get(DATA)
-            .get("artifacts")
-        )
+        return self.default_artifacts_handler.list_project_artifacts()
 
     def upload_artifact(
-        self, artifact_path: Union[Path, str, _DataFrameT], _validate: bool = True
+        self,
+        artifact_path: Union[Path, str, _DataFrameT],
+        _validate: bool = True,
+        _artifacts_handler: Optional[ArtifactsHandler] = None,
     ) -> str:
         """Resolves and uploads the data source specified in the
         model config.
@@ -224,32 +288,10 @@ class Project:
         Returns:
             A Gretel artifact key.
         """
-        if isinstance(artifact_path, str) and artifact_path.startswith("gretel_"):
-            return artifact_path
-        if isinstance(artifact_path, _DataFrameT):
-            artifact_path = BytesIO(artifact_path.to_csv(index=False).encode("utf-8"))
-            file_name = f"dataframe-{uuid.uuid4()}.csv"
-        else:
-            if _validate:
-                validate_data_source(artifact_path)
-            if isinstance(artifact_path, Path):
-                artifact_path = str(artifact_path)
-            file_name = Path(urlparse(artifact_path).path).name
-        with smart_open.open(
-            artifact_path, "rb", ignore_ext=True
-        ) as src:  # type:ignore
-            src_data = src.read()
-            art_resp = self.projects_api.create_artifact(
-                project_id=self.name, artifact=Artifact(filename=file_name)
-            )
-            artifact_key = art_resp["data"]["key"]
-            upload_resp = requests.put(
-                art_resp["data"]["url"],
-                data=src_data,
-            )
-            if upload_resp.status_code != 200:
-                upload_resp.raise_for_status()
-            return artifact_key
+        artifacts_handler = _artifacts_handler or self.default_artifacts_handler
+        if _validate and not isinstance(artifact_path, _DataFrameT):
+            artifacts_handler.validate_data_source(artifact_path)
+        return artifacts_handler.upload_project_artifact(artifact_path)
 
     def delete_artifact(self, key: str):
         """Deletes a project artifact.
@@ -257,7 +299,7 @@ class Project:
         Args:
             key: Artifact key to delete.
         """
-        return self.projects_api.delete_artifact(project_id=self.name, key=key)
+        return self.default_artifacts_handler.delete_project_artifact(key)
 
     def get_artifact_link(self, key: str) -> str:
         """Returns a link to download a project artifact.
@@ -269,58 +311,50 @@ class Project:
             A signed URL that may be used to download the given
             project artifact.
         """
-        resp = self.projects_api.download_artifact(project_id=self.name, key=key)
-        return resp[f.DATA][f.DATA][f.URL]
+        return self.default_artifacts_handler.get_project_artifact_link(key)
+
+    def get_artifact_handle(self, key: str) -> BinaryIO:
+        """Returns a reference to a remote artifact that can be used to
+        read binary data within a context manager
+
+        >>> with job.get_artifact_handle("report_json") as file:
+        ...   print(file.read())
+
+        Args:
+            key: Artifact key to download.
+
+        Returns:
+            a file like object
+        """
+        return self.default_artifacts_handler.get_project_artifact_handle(key)
 
-    # The server side API will return manifests with PENDING status if artifact processing has not completed
-    # or it will return a 404 (not found) if you immediately request the artifact before processing has even started.
-    # This is correct but not convenient.  To keep every end user from writing their own retry logic, we add some here.
-    @retry(
-        wait=wait_fixed(2),
-        stop=stop_after_attempt(15),
-        retry=retry_if_exception_type(ManifestPendingException),
-        # Instead of throwing an exception, return the pending manifest.
-        retry_error_callback=lambda retry_state: retry_state.outcome.exception().manifest,
-    )
-    @retry(
-        wait=wait_fixed(3),
-        stop=stop_after_attempt(5),
-        retry=retry_if_exception_type(ManifestNotFoundException),
-        # Instead of throwing an exception, return None.  Given that we waited for a short grace period to let the artifact become PENDING,
-        # if we are still getting 404's the key probably does not actually exist.
-        retry_error_callback=lambda retry_state: None,
-    )
     def get_artifact_manifest(
         self, key: str, retry_on_not_found: bool = True, retry_on_pending: bool = True
     ) -> dict:
-        _path = f"/projects/{self.name}/artifacts/manifest"
-        _query_params = {"key": key}
-        resp = None
-        try:
-            resp = self.projects_api.get_artifact_manifest(
-                project_id=self.name, key=key
-            )
-        except NotFoundException:
-            raise ManifestNotFoundException()
-        if retry_on_not_found and resp is None:
-            raise ManifestNotFoundException()
-        if retry_on_pending and resp is not None and resp.get("status") == "pending":
-            raise ManifestPendingException(manifest=resp)
-        return resp
+        return self.default_artifacts_handler.get_project_artifact_manifest(
+            key, retry_on_not_found, retry_on_pending
+        )
 
 
-def search_projects(limit: int = 200, query: Optional[str] = None) -> List[Project]:
+def search_projects(
+    limit: int = 200,
+    query: Optional[str] = None,
+    *,
+    session: Optional[ClientConfig] = None,
+) -> List[Project]:
     """Searches for project.
 
     Args:
         limit: The max number of projects to return.
         query: String filter applied to project names.
-        client_config: Can be used to override local Gretel config.
+        session: Can be used to override local Gretel config.
 
     Returns:
         A list of projects.
     """
-    api = get_session_config().get_api(ProjectsApi)
+    if session is None:
+        session = get_session_config()
+    api = session.get_api(ProjectsApi)
     params: Dict[str, Any] = {"limit": limit}
     if query:
         params["query"] = query
@@ -332,6 +366,7 @@ def search_projects(limit: int = 200, query: Optional[str] = None) -> List[Proje
             project_guid=p.get("guid"),
             desc=p.get("description"),
             display_name=p.get("display_name"),
+            session=session,
         )
         for p in projects.get(DATA).get(PROJECTS)
     ]
@@ -342,12 +377,15 @@ def create_project(
     name: Optional[str] = None,
     desc: Optional[str] = None,
     display_name: Optional[str] = None,
+    session: Optional[ClientConfig] = None,
 ) -> Project:
     """
     Excplit project creation. This function will simply call
     the API endpoint and will raise any HTTP exceptions upstream.
     """
-    api = get_session_config().get_api(ProjectsApi)
+    if session is None:
+        session = get_session_config()
+    api = session.get_api(ProjectsApi)
 
     payload = {}
     if name:
@@ -368,6 +406,7 @@ def create_project(
         project_guid=proj.get("guid"),
         desc=proj.get("description"),
         display_name=proj.get("display_name"),
+        session=session,
     )
 
 
@@ -377,6 +416,8 @@ def get_project(
     create: bool = False,
     desc: Optional[str] = None,
     display_name: Optional[str] = None,
+    runner_mode: Optional[Union[RunnerMode, str]] = None,
+    session: Optional[ClientConfig] = None,
 ) -> Project:
     """Used to get or create a Gretel project.
 
@@ -389,15 +430,18 @@ def get_project(
             gretel and never changes.
         desc: A short description of the project
         display_name: The main display name used in the Gretel Console for your project
-        client_config: An instance of a ClientConfig. This can be used to override any
-            default connection configuration.
+        session: the client session to use, or ``None`` to use the default session.
     Returns:
         A project instance.
     """
+    if session is None:
+        session = get_session_config()
     if not name and not create:
         raise ValueError("Must provide a name or create flag!")
 
-    api = get_session_config().get_api(ProjectsApi)
+    runner_mode = RunnerMode.parse_optional(runner_mode)
+
+    api = session.get_api(ProjectsApi)
     project = None
 
     project_args = {}
@@ -406,6 +450,8 @@ def get_project(
             project_args["description"] = desc
         if display_name:
             project_args["display_name"] = display_name
+        if runner_mode:
+            project_args["runner_mode"] = runner_mode.value
 
     if not name and create:
         resp = api.create_project(project=models.Project(**project_args))
@@ -414,7 +460,7 @@ def get_project(
     if name:
         try:
             project = api.get_project(project_id=name)
-        except UnauthorizedException:
+        except (UnauthorizedException, ForbiddenException):
             if create:
                 project_args["name"] = name
                 resp = api.create_project(project=models.Project(**project_args))
@@ -427,27 +473,40 @@ def get_project(
 
     p = project.get(DATA).get(PROJECT)
 
+    proj_runner_mode = RunnerMode.parse_optional(p.get("runner_mode"))
+    if runner_mode and runner_mode != proj_runner_mode:
+        raise GretelProjectError(
+            f"Project '{name}' exists, but it is not a {runner_mode.value} project"
+        )
+
     return Project(
         name=p.get("name"),
         project_id=p.get("_id"),
         project_guid=p.get("guid"),
         desc=p.get("description"),
         display_name=p.get("display_name"),
+        runner_mode=proj_runner_mode,
+        session=session,
     )
 
 
 @contextmanager
-def tmp_project():
+def tmp_project(*, session: Optional[ClientConfig] = None):
     """A temporary project context manager.  Create a new project
     that can be used inside of a "with" statement for temporary purposes.
     The project will be deleted from Gretel Cloud when the scope is exited.
 
+    Args:
+        session: the client session to use, or ``None`` to use the default session.
+
     Example::
 
         with tmp_project() as proj:
             model = proj.create_model_obj()
     """
-    project = get_project(create=True)
+    if session is None:
+        session = get_session_config()
+    project = get_project(create=True, session=session)
     try:
         yield project
     finally:
@@ -455,7 +514,11 @@ def tmp_project():
 
 
 def create_or_get_unique_project(
-    *, name: str, desc: Optional[str] = None, display_name: Optional[str] = None
+    *,
+    name: str,
+    desc: Optional[str] = None,
+    display_name: Optional[str] = None,
+    session: Optional[ClientConfig] = None,
 ) -> Project:
     """
     Helper function that provides a consistent experience for creating
@@ -471,18 +534,22 @@ def create_or_get_unique_project(
         desc: Description of the project.
         display_name: If None, the display name will be set equal to the value
             of ``name`` _before_ the user ID is appended.
+        session: The client session to use, or ``None`` to use the default client
+            session.
 
     NOTE:
         The ``desc`` and ``display_name`` parameters will only be used when
         the project is first created. If the project already exists, these
         params will have no affect.
     """
-    current_user_dict = get_me()
+    if session is None:
+        session = get_session_config()
+    current_user_dict = get_me(session=session)
     unique_suffix = current_user_dict["_id"][9:]
     target_name = f"{name}-{unique_suffix}"
 
     try:
-        project = get_project(name=target_name)
+        project = get_project(name=target_name, session=session)
         return project
     except GretelProjectError:
         # Project name not found
@@ -493,6 +560,9 @@ def create_or_get_unique_project(
     # Try and create the project if we coud not find it
     # originally
     project = create_project(
-        name=target_name, display_name=display_name or name, desc=desc
+        name=target_name,
+        display_name=display_name or name,
+        desc=desc,
+        session=session,
     )
     return project
