@@ -10,6 +10,7 @@ import pytest
 
 from click.testing import CliRunner, Result
 
+from gretel_client._hybrid.config import configure_hybrid_session
 from gretel_client.config import (
     _load_config,
     ClientConfig,
@@ -19,6 +20,7 @@ from gretel_client.config import (
 from gretel_client.projects.models import Model
 from gretel_client.projects.projects import get_project, Project
 from gretel_client.rest.api.projects_api import ProjectsApi
+from gretel_client.rest_v1.api.connections_api import ConnectionsApi
 
 fixtures = Path(__file__).parent / "../fixtures"
 
@@ -40,8 +42,10 @@ def runner() -> CliRunner:
 
 
 @pytest.fixture(autouse=True)
-def configure_session_client():
+def configure_session_client(request):
     """Ensures the the host client config is reset after each test."""
+    gretel_api_key = os.getenv("GRETEL_API_KEY")
+    assert gretel_api_key is not None, "GRETEL_API_KEY must be set!"
     with patch.dict(
         os.environ,
         {
@@ -50,7 +54,17 @@ def configure_session_client():
         },
         clear=True,
     ):
-        configure_session(ClientConfig.from_env())
+        config = ClientConfig.from_env()
+        configurator = configure_session
+        configure_kwargs = {}
+
+        hybrid = request.node.get_closest_marker("gretel_hybrid")
+        if hybrid:
+            configurator = configure_hybrid_session
+            configure_kwargs = hybrid.kwargs or {}
+
+        configurator(config, **configure_kwargs)
+
     yield
     configure_session(_load_config())
 
@@ -67,6 +81,11 @@ def projects_api() -> ProjectsApi:
     return get_session_config().get_api(ProjectsApi)
 
 
+@pytest.fixture
+def connections_api() -> ConnectionsApi:
+    return get_session_config().get_v1_api(ConnectionsApi)
+
+
 @pytest.fixture
 def pretrained_model_map(get_fixture: Callable) -> Dict[str, str]:
     return json.loads(get_fixture("model_fixtures.json").read_text())
