@@ -12,6 +12,8 @@ from gretel_client.evaluation.quality_report import QualityReport
 from gretel_client.evaluation.reports import _model_run_exc_message, ModelRunException
 from gretel_client.projects.projects import Project
 
+from .conftest import pytest_skip_on_windows
+
 INPUT_DF = pd.DataFrame([{"test_key": "test_value"}])
 
 
@@ -40,7 +42,9 @@ def test_manual_runner_mode_raises_an_exception(
             output_dir=tmpdir,
             runner_mode=runner_mode,
         )
-    assert str(err.value) == "Cannot use manual mode. Please use CLOUD or LOCAL."
+    assert (
+        str(err.value) == "Cannot use manual mode. Please use CLOUD, LOCAL, or HYBRID."
+    )
 
 
 @pytest.mark.parametrize(
@@ -75,6 +79,12 @@ def test_report_initialization_with_defaults(
             {
                 "evaluate": {
                     "data_source": "__tmp__",
+                    "params": {
+                        "correlation_columns": 75,
+                        "mandatory_columns": [],
+                        "sqs_report_columns": 250,
+                        "sqs_report_rows": 5000,
+                    },
                 }
             }
         ],
@@ -94,6 +104,7 @@ def test_report_initialization_with_custom_params(
         ref_data=ref_data,
         output_dir=tmpdir,
         runner_mode=RunnerMode.CLOUD,
+        mandatory_columns=["foo", "bar", "baz"],
     )
     assert report.project
     assert report.data_source
@@ -107,6 +118,12 @@ def test_report_initialization_with_custom_params(
             {
                 "evaluate": {
                     "data_source": "__tmp__",
+                    "params": {
+                        "correlation_columns": 75,
+                        "mandatory_columns": ["foo", "bar", "baz"],
+                        "sqs_report_columns": 250,
+                        "sqs_report_rows": 5000,
+                    },
                 }
             }
         ],
@@ -165,6 +182,7 @@ def test_quality_report_with_dataframes(
     assert report.peek() == {"grade": "Excellent", "raw_score": 100.0, "score": 100}
 
 
+@pytest_skip_on_windows
 @pytest.mark.parametrize(
     "runner_mode",
     [
@@ -179,8 +197,6 @@ def test_hydrated_properties(
     tmpdir: Path,
     runner_mode: RunnerMode,
 ):
-    if platform.system() == "Windows" and runner_mode == RunnerMode.LOCAL:
-        pytest.skip("Skip local runner test for Windows")
     report = QualityReport(
         project=project,
         data_source=data_source,
